package sound.audioDigitizer;

import net.multicast.McastUtil;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import java.io.IOException;

/**
 * DocJava, Inc.
 * http://www.docjava.com
 * Programmer: dlyon
 * Date: Dec 6, 2004
 * Time: 2:37:32 PM
 */
public class StreamingMcast implements Runnable {
    SourceDataLine line;
    Thread thread;
    String errStr = null;
    AudioInputStream ais = null;
    private final int bufSize = 16384;
    CapturePlayBackPanel.FormatControls formatControls = null;
    public static final int PORT = 4321;
    private McastUtil mcu = new McastUtil(PORT);

    private void sendADataGram(byte[] data,
                               int numberOfBytesRead) {
        try {
            byte b[] = new byte[numberOfBytesRead];
            System.out.println("number of bytes sent="+numberOfBytesRead);
            for (int i=0; i < b.length; i++)
                b[i]=data[i];
            mcu.sendBytes(b);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    StreamingMcast(AudioInputStream ais, CapturePlayBackPanel.FormatControls formatControls) {
        this.ais = ais;
        this.formatControls = formatControls;
        mcu.setLoopBack(true);
    }

    public void start() {
        thread = new Thread(this);
        thread.setName("broadcast");
        thread.start();

    }

    public void stop() {
        thread = null;
    }

    long delay = 1000;

    private long getDelay() {
        return delay;
    }

    private void shutDown(String message) {
        if ((errStr = message) != null) {
            System.err.println(errStr);
        }
        if (thread != null) {
            thread = null;
        }
    }

    public void run() {
        AudioFormat format = getAudioFormat();
        AudioInputStream playbackInputStream = AudioSystem.getAudioInputStream(format, ais);
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
        try {
            line = (SourceDataLine) AudioSystem.getLine(info);
            line.open(format, bufSize);
        } catch (LineUnavailableException ex) {
            shutDown("Unable to open the line: " + ex);
        }


        int numBytesRead = 0;

        // start the source data line
        line.start();
        while (thread != null) {
            int n = (int) getDelay();
            byte[] data = new byte[n];
            try {
                while ((numBytesRead = playbackInputStream.read(data)) == -1)
                    Thread.sleep(10);

                sendADataGram(data,numBytesRead);
                //threadedAudioPlayer();
                //playAudio(numBytesRemaining, data);
            } catch (Exception e) {
                shutDown("Error during playback: " + e);
                break;
            }
        }
        cleanUp();
    }

    public void threadedAudioPlayer() {
        Thread t = new Thread(new Runnable() {
            public void run() {
                try {
                    while (true) {
                        byte b[] = mcu.getBytes();
                        System.out.println("rcving bytes!!="+b.length);
                        playAudio(b.length, b);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        t.start();
    }

    private void playAudio(int numBytesRemaining, byte[] data) {
        while (numBytesRemaining > 0)
            numBytesRemaining -= line.write(data, 0, numBytesRemaining);
    }

    private void cleanUp() {
        // we reached the end of the stream.  let the data play out, then
        // stop and close the line.
        if (thread != null) {
            line.drain();
        }
        line.stop();
        line.close();
        line = null;
        shutDown(null);
    }

    private AudioFormat getAudioFormat() {
        // make sure we have something to play
        if (ais == null)
            shutDown("No loaded audio to play back");

        // get an AudioInputStream of the desired format for playback
        AudioFormat format = formatControls.getFormat();
        return format;
    }
}
